package lauegramm;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;


/**
 * Created on May 12, 2008
 * @author Michael Leitner
 */
/**
 * This file is part of lauegramm.
 * lauegramm is a Java application/applet, which renders a crystal in real space
 * and the Laue back-scattering obtained by a white beam,
 * together with a graphical user interface.
 * Copyright 2008, Michael Leitner
 *
 * As stated below, you are free to do whatever you want with this program, as long as
 * it conforms to the licence, but I would be glad to hear if someone found it useful,
 * so please tell me if so under michael.leitner@univie.ac.at
 *
 * lauegramm is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * lauegramm is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class LGFrame extends Frame implements ActionListener,ItemListener {
    Lattice lattice;
    Orientation orientation;
    RealCanvas realCanvas;
    LaueCanvas laueCanvas;
    ControlPanel controlPanel=new ControlPanel(this,this);
    
    public static void main(String[] args) {
        LGFrame grafik=new LGFrame();
    }
    
    public LGFrame() {
        super("Lauegramm Simulation");
        addWindowListener(new WindowClosingAdapter());
        setBackground(Color.lightGray);
        setLayout(new FlowLayout());
        orientation=new Orientation();
        lattice = new Lattice(orientation);
        laueCanvas=new LaueCanvas(lattice,orientation);
        realCanvas=new RealCanvas(lattice,orientation,laueCanvas);
        add(realCanvas);                       
        add(controlPanel);              
        add(laueCanvas);           
        actionPerformed(new ActionEvent(new Button(),0,""));
        controlPanel.itemStateChanged();
        pack();
        setVisible(true);
    }

    public void actionPerformed(ActionEvent event) {
        int i;
        String string=controlPanel.basisCBG.getSelectedCheckbox().getLabel();
        if (string.equals("primitive"))
            i=0;
        else if (string.equals("body-centered"))
            i=1;
        else if (string.equals("face-centered"))
            i=2;
        else if (string.equals("base-centered"))
            i=3;
        else if (string.equals("diamond"))
            i=4;
        else 
            i=5;
        lattice.choose(controlPanel.latticeType.getSelectedItem(),i,Double.parseDouble(controlPanel.textFields[0].getText()),Double.parseDouble(controlPanel.textFields[1].getText()),Double.parseDouble(controlPanel.textFields[2].getText()),Double.parseDouble(controlPanel.textFields[3].getText()),Double.parseDouble(controlPanel.textFields[4].getText()),Double.parseDouble(controlPanel.textFields[5].getText()),Integer.parseInt(controlPanel.textFields[6].getText()),Integer.parseInt(controlPanel.textFields[7].getText()),Integer.parseInt(controlPanel.textFields[8].getText()));
        realCanvas.zoomFactor=Double.parseDouble(controlPanel.textFields[9].getText())/10.;
        realCanvas.radius=Double.parseDouble(controlPanel.textFields[10].getText());
        realCanvas.repaint();
        
    }
    
    public void itemStateChanged(ItemEvent event) {
        if (event.getStateChange()==ItemEvent.DESELECTED)
            remove(2);
        else if (event.getStateChange()==ItemEvent.SELECTED)
            add(laueCanvas,2);
        pack();
    }
}

class ControlPanel extends Panel implements ItemListener {
    Choice latticeType=new Choice();
    TextField[] textFields=new TextField[11];
    Label[] labels=new Label[8];
    CheckboxGroup basisCBG = new CheckboxGroup();
    Checkbox[] basisCB =new Checkbox[6];
    Checkbox lauegramm=new Checkbox();
    Panel basisPanel=new Panel(null);
    Panel paramPanel=new Panel(null);
    Panel latticePanel=new Panel(null);
    Panel drawPanel=new Panel(null);
    
    public ControlPanel(ActionListener al,ItemListener il) {
        int i;
        setSize(150,260);
        setLayout(new BorderLayout());
        basisPanel.setSize(150,80);
        paramPanel.setSize(150,120);
        latticePanel.setSize(150,40);
        drawPanel.setSize(150,60);  
        lauegramm=new Checkbox("Lauegramm?",true);
        lauegramm.addItemListener(il);        
        lauegramm.setBounds(0,0,150,20);
        latticePanel.add(lauegramm);      
        latticeType.add("cubic");
        latticeType.add("tetragonal");
        latticeType.add("orthorhombic");
        latticeType.add("hexagonal");
        latticeType.add("trigonal");
        latticeType.add("monoclinic");
        latticeType.add("triclinic");
        latticeType.setBounds(0,20,100,20);
        latticePanel.add(latticeType);      
        latticeType.addItemListener(this);
        latticeType.select("cubic");
        Button newLattice=new Button("update");     
        newLattice.setBounds(100,20,50,20);   
        latticePanel.add(newLattice);
        add(latticePanel,BorderLayout.NORTH);
        add(paramPanel,BorderLayout.CENTER);
        Panel subPanel=new Panel(new BorderLayout());
        subPanel.add(basisPanel,BorderLayout.NORTH);
        subPanel.add(drawPanel,BorderLayout.SOUTH);
        add(subPanel,BorderLayout.SOUTH);
        newLattice.addActionListener(al);  
        for (i=0;i<2;i++) textFields[i]=new TextField("1.0",5);
        textFields[2]=new TextField("1.633",5);
        for (i=3;i<6;i++) textFields[i]=new TextField("90.0",5);
        for (i=6;i<9;i++) textFields[i]=new TextField("1",5);
        textFields[9]=new TextField("1.0",5);
        textFields[10]=new TextField("0.15",5);
        labels[0]=new Label("a: ");     
        labels[1]=new Label("b: ");     
        labels[2]=new Label("c: ");     
        labels[3]=new Label("alpha: ");
        labels[4]=new Label("beta: ");
        labels[5]=new Label("gamma: ");
        labels[6]=new Label("Zoom: ");
        labels[7]=new Label("Atomic Radius: ");
        for (i=0;i<6;i++) {
            labels[i].setBounds(0,20*i,50,20);
            textFields[i].setBounds(50,20*i,100,20);
        }
        for (i=0;i<3;i++) {
            textFields[i+6].setBounds(50*i,0,50,20);
            drawPanel.add(textFields[i+6]);
        }
        labels[6].setBounds(0,20,90,20);
        drawPanel.add(labels[6]);
        textFields[9].setBounds(90,20,60,20);
        drawPanel.add(textFields[9]);
        labels[7].setBounds(0,40,90,20);
        drawPanel.add(labels[7]);
        textFields[10].setBounds(90,40,60,20);
        drawPanel.add(textFields[10]);
        basisCB[0]=new Checkbox("primitive",basisCBG,true);
        basisCB[1]=new Checkbox("body-centered",basisCBG,false);
        basisCB[2]=new Checkbox("face-centered",basisCBG,false);
        basisCB[3]=new Checkbox("base-centered",basisCBG,false);
        basisCB[4]=new Checkbox("diamond",basisCBG,false);
        basisCB[5]=new Checkbox("close-packed",basisCBG,false);
        for (i=0;i<3;i++) {
            basisCB[i].setBounds(0,20*i,100,20);
        }
        for (i=3;i<6;i++) {
            basisCB[i].setBounds(0,60,100,20);
        }
    }   
    
    public void itemStateChanged(ItemEvent event) {
        itemStateChanged();
    }
    
    public void itemStateChanged() {
        int i;
        boolean[] check=new boolean[12];
        String string=latticeType.getSelectedItem();
        check[0]=true;
        check[6]=true;
        if (!((string.equals("cubic"))||(string.equals("trigonal")))) {
            check[2] = true;
            if (!((string.equals("hexagonal"))||(string.equals("tetragonal")))) {
                check[1]=true;
                if (!string.equals("orthorhombic")) {
                    check[4]=true;
                    if (string.equals("triclinic")) {
                        check[3]=true;
                        check[5]=true;                        
                    }
                }
            }
        }
        if (string.equals("trigonal")) check[3]=true;
        
        if ((string.equals("cubic"))||(string.equals("tetragonal"))||(string.equals("orthorhombic")))
            check[7]=true;
        if ((string.equals("monoclinic"))||(string.equals("orthorhombic")))
            check[9]=true;
        if ((string.equals("cubic"))||(string.equals("orthorhombic")))
            check[8]=true;
        if (string.equals("cubic"))
            check[10]=true;
        if (string.equals("hexagonal"))
            check[11]=true;
         
        for (i=0;i<6;i++) {
            paramPanel.remove(labels[i]);
            paramPanel.remove(textFields[i]);
            if (check[i]) {
                paramPanel.add(labels[i]);
                paramPanel.add(textFields[i]);
            }
        }
        for (i=0;i<6;i++) {
            basisPanel.remove(basisCB[i]);
            if (check[i+6]) basisPanel.add(basisCB[i]);
            else if (basisCB[i].getState())
                basisCB[0].setState(true);
        }
        validate();
    }
}


class Position {
    double x,y,z;
    
    public Position (double x,double y, double z) {
        this.x=x;
        this.y=y;
        this.z=z;
    }
}


class Orientation {
    private double[][] matrix=new double[3][3];
    private double[][] oldMatrix=new double[3][3];
    
    public Orientation() {
        matrix[0][0]=1.;
        matrix[1][1]=1.;
        matrix[2][2]=1.;
        oldMatrix[0][0]=1.;
        oldMatrix[1][1]=1.;
        oldMatrix[2][2]=1.;
    }
    
    public void rotate(double e1,double e2,double e3, double theta) {
        int i,j,k;
        double[][] rotMatrix=new double[3][3];
        double[][] dumMatrix=new double[3][3];
        double costh=Math.cos(theta);
        double sinth=Math.sin(theta);
        rotMatrix[0][0]=costh+(1.-costh)*e1*e1;
        rotMatrix[0][1]=(1.-costh)*e1*e2+sinth*e3;
        rotMatrix[0][2]=(1.-costh)*e1*e3-sinth*e2;
        rotMatrix[1][0]=(1.-costh)*e1*e2-sinth*e3;
        rotMatrix[1][1]=costh+(1.-costh)*e2*e2;
        rotMatrix[1][2]=(1.-costh)*e3*e2+sinth*e1;
        rotMatrix[2][0]=(1.-costh)*e1*e3+sinth*e2;
        rotMatrix[2][1]=(1.-costh)*e3*e2-sinth*e1;
        rotMatrix[2][2]=costh+(1.-costh)*e3*e3;
        for (i=0;i<3;i++) for (j=0;j<3;j++) for (k=0;k<3;k++) {
            dumMatrix[i][j]+=rotMatrix[i][k]*oldMatrix[k][j];
        }
        matrix=dumMatrix;
        //System.out.println(matrix[0][0]+" "+matrix[0][1]+" "+matrix[0][2]+" ");
        //System.out.println(matrix[1][0]+" "+matrix[1][1]+" "+matrix[1][2]+" ");
        //System.out.println(matrix[2][0]+" "+matrix[2][1]+" "+matrix[2][2]+" ");
    }
    
    public Position[] renderPos(Position[] dumPos) {
        int i;
        Position[] renderPos=new Position[dumPos.length];
        for (i=0;i<dumPos.length;i++) 
            renderPos[i]=new Position(matrix[0][0]*dumPos[i].x+matrix[0][1]*dumPos[i].y+matrix[0][2]*dumPos[i].z,matrix[1][0]*dumPos[i].x+matrix[1][1]*dumPos[i].y+matrix[1][2]*dumPos[i].z,matrix[2][0]*dumPos[i].x+matrix[2][1]*dumPos[i].y+matrix[2][2]*dumPos[i].z);
        return renderPos;
    }
    
    public void setMatrix() {
        oldMatrix=matrix;
    }
    
}

class Lattice {
    //double a,b,c; //a in x-richtung, b in y, c in z
    //double alpha,beta,gamma; //winkel zw. bc, ca, ab
    public Position[] realLattice=new Position[0];
    public Position[] reciprocalLattice=new Position[0];
    public Position[] basis;
    public double[][] reciprocalLatticeVect=new double[3][3];
    public double[] reciprocalLatticeMagn;
    private Orientation orientation;
    
    public Lattice(Orientation orientation) {
        this.orientation=orientation;
    }
    
    public void choose(String latticeType, int basisType,double a, double b, double c, double alpha, double beta, double gamma,int anzx,int anzy,int anzz) {
        switch (basisType) {
            case 0: {                
                basis = new Position[1];
                basis[0] = new Position(0., 0., 0.);
                break;
            }
            case 1: {                
                basis = new Position[2];
                basis[0] = new Position(0., 0., 0.);
                basis[1] = new Position(0.5, 0.5, 0.5);
                break;
            }
            case 2: {                
                basis = new Position[4];
                basis[0] = new Position(0., 0., 0.);
                basis[1] = new Position(0.5, 0.5, 0.);
                basis[2] = new Position(0.5, 0., 0.5);
                basis[3] = new Position(0., 0.5, 0.5);
                break;
            }
            case 3: {                
                basis = new Position[2];
                basis[0] = new Position(0., 0., 0.);
                basis[1] = new Position(0.5, 0.5, 0.);
                break;
            }
            case 4: {               
                basis = new Position[8];
                basis[0] = new Position(0., 0., 0.);
                basis[1] = new Position(0.5, 0.5, 0.);
                basis[2] = new Position(0.5, 0., 0.5);
                basis[3] = new Position(0., 0.5, 0.5);
                basis[4] = new Position(0.25, 0.25, 0.25);
                basis[5] = new Position(0.75, 0.75, 0.25);
                basis[6] = new Position(0.75, 0.25, 0.75);
                basis[7] = new Position(0.25, 0.75, 0.75);
                break;
            }
            case 5: {                
                basis = new Position[2];
                basis[0] = new Position(0., 0., 0.);
                basis[1] = new Position(1./3., 1./3., 0.5);
            }
        }
        if (latticeType.equals("cubic")) {
            construct(anzx, anzy, anzz, a, a, a, Math.PI / 2., Math.PI / 2., Math.PI / 2., basis);
        } else if (latticeType.equals("tetragonal")) {
            construct(anzx, anzy, anzz, a, a, c, Math.PI / 2., Math.PI / 2., Math.PI / 2., basis);
        } else if (latticeType.equals("orthorhombic")) {
            construct(anzx, anzy, anzz, a, b, c, Math.PI / 2., Math.PI / 2., Math.PI / 2., basis);
        } else if (latticeType.equals("hexagonal")) {
            construct(anzx, anzy, anzz, a, a, c, Math.PI / 2., Math.PI / 2., Math.PI / 3., basis);
        } else if (latticeType.equals("trigonal")) {
            construct(anzx, anzy, anzz, a, a, a, alpha*Math.PI / 180., alpha*Math.PI / 180., alpha*Math.PI / 180., basis);
        } else if (latticeType.equals("monoclinic")) {
            construct(anzx, anzy, anzz, a, b, c, Math.PI / 2., beta*Math.PI / 180., Math.PI / 2., basis);
        } else if (latticeType.equals("triclinic")) {
            construct(anzx, anzy, anzz, a, b, c, alpha*Math.PI / 180., beta*Math.PI / 180., gamma*Math.PI / 180., basis);
        } 
    }
    
    private void construct(int anzx,int anzy, int anzz, double a, double b, double c, double alpha, double beta, double gamma, Position[] basis) {
        int i, j, k, l;
        double dum1,dum2,dum3,dum,dumr,dumi,vol;
        double[][] realLatticeVect=new double[3][3];//1.koord:welcher vektor
        realLatticeVect[0][0]=a;
        realLatticeVect[1][0]=b*Math.cos(gamma);
        realLatticeVect[1][1]=b*Math.sin(gamma);
        dum1=Math.cos(beta);
        realLatticeVect[2][0]=c*dum1;
        dum2=(Math.cos(alpha)-dum1*Math.cos(gamma))/Math.sin(gamma);
        realLatticeVect[2][1]=c*dum2;
        realLatticeVect[2][2]=c*Math.sqrt(1-dum1*dum1-dum2*dum2);        
        ArrayList dumArray=new ArrayList();
         for (i = 0; i <= anzx; i++) 
            for (j = 0; j <= anzy; j++) 
                for (k = 0; k <= anzz; k++) 
                    for (l=0;l<basis.length;l++) 
                        if (((i<anzx)||(basis[l].x==0.))&&((j<anzy)||(basis[l].y==0.))&&((k<anzz)||(basis[l].z==0.))) {
                            dum1=(i-anzx/2.+basis[l].x)*realLatticeVect[0][0]+(j-anzy/2.+basis[l].y)*realLatticeVect[1][0]+(k-anzz/2.+basis[l].z)*realLatticeVect[2][0];
                            dum2=(i-anzx/2.+basis[l].x)*realLatticeVect[0][1]+(j-anzy/2.+basis[l].y)*realLatticeVect[1][1]+(k-anzz/2.+basis[l].z)*realLatticeVect[2][1];
                            dum3=(i-anzx/2.+basis[l].x)*realLatticeVect[0][2]+(j-anzy/2.+basis[l].y)*realLatticeVect[1][2]+(k-anzz/2.+basis[l].z)*realLatticeVect[2][2];                    
                            dumArray.add(new Position(dum1,dum2,dum3));
                }
        realLattice=(Position[])dumArray.toArray(new Position[1]);   
        
        for (i=0;i<3;i++)
            for (j=0;j<3;j++) {
                reciprocalLatticeVect[i][j]=realLatticeVect[(i+1)%3][(j+1)%3]*realLatticeVect[(i+2)%3][(j+2)%3]-realLatticeVect[(i+1)%3][(j+2)%3]*realLatticeVect[(i+2)%3][(j+1)%3];
            }
        vol=0;
        for (i=0;i<3;i++) vol+=reciprocalLatticeVect[0][i]*realLatticeVect[0][i];
        vol=Math.exp(Math.log(vol/basis.length)/3);
        //System.out.println(vol);
        for (i=0;i<3;i++) {
            dum1=0.;
            for (j=0;j<3;j++) dum1+=reciprocalLatticeVect[i][j]*realLatticeVect[i][j];
            for (j=0;j<3;j++) reciprocalLatticeVect[i][j]/=dum1;
        }
        dumArray=new ArrayList();
        ArrayList dumArray2=new ArrayList();
         for (i = -6; i < 7; i++) 
            for (j = -6; j < 7; j++) 
                for (k = -6; k < 7; k++) {
                    dum1=i*reciprocalLatticeVect[0][0]+j*reciprocalLatticeVect[1][0]+k*reciprocalLatticeVect[2][0];
                    dum2=i*reciprocalLatticeVect[0][1]+j*reciprocalLatticeVect[1][1]+k*reciprocalLatticeVect[2][1];
                    dum3=i*reciprocalLatticeVect[0][2]+j*reciprocalLatticeVect[1][2]+k*reciprocalLatticeVect[2][2];
                    dumr=0.;        
                    dumi=0.;        
                    for (l=0;l<basis.length;l++) {
                        dum=2.*Math.PI*(basis[l].x*dum1+basis[l].y*dum2+basis[l].z*dum3);
                        dumr+=Math.cos(dum);
                        dumi+=Math.sin(dum);
                    }                    
                    dum=Math.sqrt(dum1*dum1+dum2*dum2+dum3*dum3);
                    dumr=(dumr*dumr+dumi*dumi)/(basis.length*basis.length)/(10.+.2*dum*dum*dum*dum*vol*vol*vol*vol);
                    if ((dum>0.)&&(dumr>.005)) {
                        dumArray.add(new Position(dum1/dum,dum2/dum,dum3/dum));
                        dumArray2.add(new Double(dumr));
                    }
                }
        reciprocalLattice=(Position[])dumArray.toArray(new Position[1]);
        reciprocalLatticeMagn=new double[reciprocalLattice.length];
        for (i=0;i<reciprocalLatticeMagn.length;i++) 
            reciprocalLatticeMagn[i]=((Double)dumArray2.get(i)).doubleValue();
        //System.out.println(reciprocalLatticeMagn.length);
    }    
}


class RealCanvas extends Canvas {

    double pixperm=4000.;
    BufferedImage renderImage;
    DataBufferInt renderBuffer;
    WritableRaster renderRaster; 
    Orientation orientation;
    Lattice lattice;
    LaueCanvas laueCanvas;
    int sizex=400,sizey=400;
    private double[] bookkeep=new double[sizex*sizey];
    int[][] colorSphere=new int[101][101];   
    double[] lightVector={.707,.5,.5};    
    //double r=0.6,g=0.7,b=.75,diffuse=.6;
    double r=0.,g=0.,b=1.,diffuse=.35;
    double zoomFactor,radius;
    
    public RealCanvas(Lattice lattice,Orientation orientation,LaueCanvas laueCanvas) {
        int i,j;
        double x,y,z,dum1,dum2,red,green,blue;
        setSize(sizex, sizey);
        MouseDragAdapter mouseDragAdapter=new MouseDragAdapter(this);
        addMouseListener(mouseDragAdapter);
        addMouseMotionListener(mouseDragAdapter);
        addMouseWheelListener(mouseDragAdapter);
        this.orientation=orientation;
        this.lattice=lattice;
        this.laueCanvas=laueCanvas;
        renderImage = new BufferedImage(sizex,sizey,BufferedImage.TYPE_INT_RGB);
        renderBuffer=(DataBufferInt)renderImage.getRaster().getDataBuffer();
        renderRaster = renderImage.getRaster();
        
        
        for (i=0;i<101;i++) {for (j=0;j<101;j++) {
            x=(i-50)/50.;
            y=(j-50)/50.;
            if (x*x+y*y<1.) {
                red=0;
                green=0;
                blue=0;
                z=Math.sqrt(1.-x*x-y*y);
                dum1=lightVector[0]*x+lightVector[1]*y+lightVector[2]*z;
                dum2=Math.pow(Math.max(0.,2*z*dum1-lightVector[2]),40.); 
                dum1=Math.max(0.,dum1);
                red=2./Math.PI*Math.atan(r*(Math.tan(diffuse*Math.PI/2.)+Math.tan(dum1*Math.PI/2.))+Math.tan(dum2*Math.PI/2.));
                green=2./Math.PI*Math.atan(g*(Math.tan(diffuse*Math.PI/2.)+Math.tan(dum1*Math.PI/2.))+Math.tan(dum2*Math.PI/2.));
                blue=2./Math.PI*Math.atan(b*(Math.tan(diffuse*Math.PI/2.)+Math.tan(dum1*Math.PI/2.))+Math.tan(dum2*Math.PI/2.));
                colorSphere[i][j]=((int)(red*255.)*256+(int)(green*255.))*256+(int)(blue*255.);
            }
            //System.out.print(colorSphere[i][j]+" ");
        }
        //System.out.println();
        }
    }
    
    public void dragged(int dx1,int dy1,int dx2,int dy2) {
        int x1=dx1-this.getSize().width/2;
        int x2=dx2-this.getSize().width/2;
        int y1=dy1-this.getSize().height/2;
        int y2=dy2-this.getSize().height/2;
        double z1=.2*pixperm;
        double z2=.2*pixperm;
        
        //System.out.println(x1);
        double theta,dum;
        double dumx,dumy,dumz;
        theta=Math.acos((x1*x2+y1*y2+z1*z2)/Math.sqrt((x1*x1+y1*y1+z1*z1)*(x2*x2+y2*y2+z2*z2)));
        //System.out.println(theta);
        dumx=y1*z2-y2*z1;
        dumy=z1*x2-z2*x1;
        dumz=x1*y2-x2*y1;
        dum=Math.sqrt(dumx*dumx+dumy*dumy+dumz*dumz);
        orientation.rotate(dumx/dum,dumy/dum,dumz/dum,theta);
        repaint();
    }
    
    public void rotated(int x,int y,int clicks) {
        x=x-getSize().width/2;
        y=y-getSize().height/2;
        int z=(int)(.2*pixperm);
        double dum=Math.sqrt((double)(x*x+y*y+z*z));
        orientation.rotate(-x/dum,-y/dum,z/dum,clicks/15.);
        repaint();
    }
    
    public void setMatrix() {
        orientation.setMatrix();
    }
    
    //@Override
    public void paint(Graphics g) {
        int i,j,k;
        double dum,rad;
        int intrad,posx,posy;
        Position[] renderPos;
        for (i=0;i<sizex*sizey;i++) if(bookkeep[i]<100.) {
            bookkeep[i]=100.;
            renderBuffer.setElem(i,0);
            //renderBuffer.setElem(i, (255*256+255)*256+255);
        }
        renderPos=orientation.renderPos(lattice.realLattice);
        for (i=0;i<renderPos.length;i++) {
            dum=.5/(1.+zoomFactor*renderPos[i].z);
            rad=radius*zoomFactor*pixperm*dum;
            intrad=(int)rad;
            posx=(int)(sizex/2.+pixperm*zoomFactor*renderPos[i].x*dum);
            posy=(int)(sizey/2.+pixperm*zoomFactor*renderPos[i].y*dum);
            for (j=-intrad;j<=intrad;j++) for (k=-intrad;k<=intrad;k++)
                if ((posy+k>=0)&&(posy+k<sizey)&&(posx+j>=0)&&(posx+j<sizex)&&(bookkeep[(posy+k)*sizex+posx+j]>renderPos[i].z)&&(j*j+k*k<intrad*intrad)) {
                    renderBuffer.setElem((posy+k)*sizex+posx+j, colorSphere[(int)(j/rad*50.)+50][(int)(k/rad*50)+50]);
                    bookkeep[(posy+k)*sizex+posx+j]=renderPos[i].z;
                }
        }
        g.drawImage(renderImage,0,0,null);
     }
    
    //@Override
    public void update(Graphics g) {
        paint(g);
        laueCanvas.repaint();
    }

}

class LaueCanvas extends Canvas {

    double pixperm=4000.;
    BufferedImage renderImage;
    DataBufferInt renderBuffer;
    WritableRaster renderRaster; 
    Orientation orientation;
    Lattice lattice;
    int sizex=400,sizey=400;  
    double[] dumArray=new double[sizex*sizey];
    
    public LaueCanvas(Lattice lattice,Orientation orientation) {
        setSize(sizex, sizey);
        this.orientation=orientation;
        this.lattice=lattice;
        renderImage = new BufferedImage(sizex,sizey,BufferedImage.TYPE_INT_RGB);
        renderBuffer=(DataBufferInt)renderImage.getRaster().getDataBuffer();
        renderRaster = renderImage.getRaster();
    }
    
    
    //@Override
    public void paint(Graphics g) {
        int i,j,k;
        int posx,posy;
        double dum;
        Position[] renderPos=orientation.renderPos(lattice.reciprocalLattice);
        
        for (i=0;i<sizex*sizey;i++) dumArray[i]=0.;
        
        for (k=0;k<renderPos.length;k++) {
            if ((dum=renderPos[k].z)>.82) {
                dum=2.*dum*dum-1.;
                posx=sizex/2-(int)(100.*renderPos[k].x/dum);
                posy=sizey/2-(int)(100.*renderPos[k].y/dum);
                dum=lattice.reciprocalLatticeMagn[k];
                for (i=-7;i<8;i++) for (j=-7;j<8;j++)
                    dumArray[(posy+i)*sizex+posx+j]+=dum*Math.exp(-(i*i+j*j)/10.);
            }
        }
        
        for (i=0;i<sizex*sizey;i++) {
            j=(int)(255.*Math.min(1.,20.*dumArray[i]));
            renderBuffer.setElem(i,(j*256+j)*256+j);
        }
        for (i=-3;i<=3;i++) renderBuffer.setElem(sizex*(sizey/2)+sizex/2+i, (255*256+255)*256+255);
        for (i=-3;i<=3;i++) renderBuffer.setElem(sizex*(sizey/2+i)+sizex/2, (255*256+255)*256+255);
        //renderBuffer.setElem(sizex*(sizey/2)+sizex/2,256*256*255);
        g.drawImage(renderImage,0,0,null);
     }
    
    //@Override
    public void update(Graphics g) {
        paint(g);
    }

}

class MouseDragAdapter extends MouseAdapter implements MouseMotionListener,MouseWheelListener {
    int dumx,dumy;
    RealCanvas realCanvas;
    
    public MouseDragAdapter(RealCanvas realCanvas) {
        this.realCanvas=realCanvas;
    }
    
    //@Override
    public void mousePressed(MouseEvent e) {
        dumx=e.getX();
        dumy=e.getY();
    }
    
    //@Override
    public void mouseDragged(MouseEvent e) {
        //System.out.println(dumx+" "+dumy+" "+e.getX()+" "+e.getY());
        if ((dumx!=e.getX())||(dumy!=e.getY())) realCanvas.dragged(dumx,dumy,e.getX(),e.getY());
    }
    
    //@Override
    public void mouseReleased(MouseEvent e) {
        realCanvas.setMatrix();
    }
    
    //@Override
    public void mouseWheelMoved(MouseWheelEvent e){
        realCanvas.setMatrix();
        dumx=e.getX();
        dumy=e.getY();
        realCanvas.rotated(dumx,dumy,e.getWheelRotation());
        realCanvas.setMatrix();
    }
    
}

class WindowClosingAdapter extends WindowAdapter
{    
    //@Override
    public void windowClosing(WindowEvent event)
    {
        event.getWindow().setVisible(false);
        event.getWindow().dispose();
        System.exit(0);
    }
}